/**
  @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_SIMPLE_CONTROLLER_PLUGIN_SIMPLE_CONTROLLER_ITEM_H_INCLUDED
#define CNOID_SIMPLE_CONTROLLER_PLUGIN_SIMPLE_CONTROLLER_ITEM_H_INCLUDED

#include <cnoid/ControllerItem>
#include <cnoid/Body>
#include <QLibrary>

namespace cnoid {

    class SimpleController;
    class MessageView;

    class SimpleControllerItem : public ControllerItem
    {
    public:
        SimpleControllerItem();
        SimpleControllerItem(const SimpleControllerItem& org);
        virtual ~SimpleControllerItem();
        
        void setControllerDllName(const std::string& name);
        
        virtual bool initialize(Body* body, double worldTimeStep);
        virtual double timeStep();
        virtual void input();
        virtual bool control();
        virtual void output();
        virtual void finalize();
        
    protected:
        virtual void onDisconnectedFromRoot();
        virtual ItemPtr doDuplicate() const;
        virtual void doPutProperties(PutPropertyFunction& putProperty);
        virtual bool store(Archive& archive);
        virtual bool restore(const Archive& archive);
        
      private:
        QLibrary controllerDll;
        bool doReloading;
        std::string controllerDllName;
        std::string controllerDllFileName;
        SimpleController* controller;
        Body* simulationBody;
        BodyPtr internalBody;
        double timeStep_;
        MessageView* mv;

        void unloadController();
        bool onReloadingChanged(bool on);
    };
        
    typedef boost::intrusive_ptr<SimpleControllerItem> SimpleControllerItemPtr;
}

#endif
